from copy import deepcopy
from datetime import date
from pathlib import Path
from typing import Dict

import click

from yearmaps.constant import Configs
from yearmaps.utils.util import dict_hash, str_hash


class Task:
    def __init__(self,
                 name: str,
                 context: click.Context,
                 command: click.Command,
                 global_options: Dict,
                 options: Dict):
        self.name = name
        self.command = command
        self.command_options = options
        self.context = deepcopy(context)
        self.global_config: Configs = self.context.obj
        for key, value in global_options.items():
            setattr(self.global_config, key, value)
        self.global_config.output = str(Path(
            self.global_config.output) / f"{self.task_hash()}.{self.global_config.file_type}")

    def run(self, force=False):
        if self.should_run() or force:
            self.context.invoke(self.command, **self.command_options)

    def should_run(self) -> bool:
        if self.context.obj.mode == 'year':
            return date.today().year == self.global_config.year
        return True

    def task_hash(self) -> str:
        command_options_hash = dict_hash(self.command_options)
        global_options_hash = self.global_config.hash()
        return str_hash(command_options_hash, global_options_hash, self.command.name)

    def cache_path(self) -> Path:
        return Path(self.global_config.output)

    def task_name(self) -> str:
        return f"{self.command.name} {self.command_options} {self.global_config} \n{self.task_hash()}"

    def update_cache(self):
        click.echo(f"Updating cache for {self.global_config} {self.command_options}")
        self.run(force=True)

    def ensure_cache(self, quiet=False):
        if not quiet:
            click.echo(f"Ensuring cache for {self.global_config} {self.command_options}")
        self.run(force=True)



import datetime
import json
from abc import ABC, abstractmethod
from contextlib import contextmanager
from pathlib import Path
from typing import Any, List, Callable, Union, Dict

import click
import numpy as np

from yearmaps.constant import Configs
from yearmaps.utils import YearData


class ProviderInfo(ABC):

    # Label formatter
    @staticmethod
    def label_format(value: Union[int, float]) -> str:
        return f"{value}"

    # Component unit.
    @property
    @abstractmethod
    def unit(self) -> Union[str, Callable]:
        pass

    @property
    def value_type(self) -> Callable:
        return int

    @staticmethod
    def analysis(data: np.ndarray):
        return np.nansum(data)

    # Provider name
    @property
    def name(self) -> str:
        return self.id.upper()

    # Provider id
    @property
    @abstractmethod
    def id(self) -> str:
        pass

    # Render color, should have 10 items
    @property
    @abstractmethod
    def color(self) -> List[str]:
        pass

    # Global group options
    options: Configs = None


class ProviderInterface(ABC):
    # Get raw utils from utils source.
    @abstractmethod
    def access(self) -> Any:
        pass

    # Parse raw utils to standard format.
    @abstractmethod
    def process(self, raw: Any) -> YearData:
        pass

    def init(self):
        pass

    # Register command to click
    @staticmethod
    @abstractmethod
    def command():
        pass


class ProviderUtils(ProviderInfo, ABC):
    # Read cache
    def __read_data_file(self) -> Dict:
        data_file = Path(self.options.data_dir) / f"{self.id}.json"
        if not data_file.exists():
            return {}
        with open(data_file, "r", encoding='UTF-8') as f:
            data = json.load(f)
        return data

    # Write cache
    def __write_data_file(self, cache: Any):
        cache_file = Path(self.options.data_dir) / f"{self.id}.json"
        with open(cache_file, "w", encoding='UTF-8') as f:
            json.dump(cache, f)

    @contextmanager
    def data_file(self):
        data = self.__read_data_file()
        yield data
        self.__write_data_file(data)

    # Judge a date should be rendered or not.
    def is_date_valid(self, date: datetime.date):
        return self.start_date() <= date <= self.end_date()

    # The start date under current mode
    def start_date(self):
        mode = self.options.mode
        if mode == 'year':
            return datetime.date(self.options.year, 1, 1)
        return datetime.date.today() - datetime.timedelta(days=366)

    # The end date under current mode
    def end_date(self):
        mode = self.options.mode
        if mode == 'year':
            return datetime.date(self.options.year, 12, 31)
        return datetime.date.today()

    def echo(self, msg: str):
        if not self.options.server:
            click.echo(msg)


def date_range(start_date: datetime.date, end_date: datetime.date):
    for n in range(int((end_date - start_date).days) + 1):
        yield start_date + datetime.timedelta(n)



from abc import ABC
from datetime import datetime
from typing import Any

import click
import requests

from yearmaps.impl.provider import Provider
from yearmaps.utils import YearData
from yearmaps.utils.colors import orange
from yearmaps.utils.error import ProviderError

ENDPOINT_URL = "https://learnywhere.cn/bb/dashboard/profile/search?userId={user_id}"


class BBDCProvider(Provider, ABC):
    name = "不背单词"
    id = "bbdc"
    color = orange

    def __init__(self, uid: str):
        self.uid = uid

    @staticmethod
    @click.command('bbdc', help="不背单词")
    @click.option('--id', '-i', 'uid', type=str, required=True, help='不背单词用户 ID')
    @click.option('--type', '-t', 'gtype', type=click.Choice(['time', 'word']), default='time', help='图数据类型')
    @click.pass_context
    def command(ctx: click.Context, uid: str, gtype: str):
        if gtype == 'time':
            provider = BBDCTimeProvider(uid)
        elif gtype == "word":
            provider = BBDCWordProvider(uid)
        else:
            raise ProviderError(f"Unknown type {gtype}")
        provider.render(ctx.obj)

    def access(self):

        """
            data structure
            id: user-id
            utils:
              2021-1-1:
                learn: 20
                time: 5
                review: 20
        """
        with self.data_file() as data:
            if not data:
                data = {
                    "id": self.uid,
                    "utils": {}
                }
            if data.get("id") != self.uid:
                raise ProviderError("user_id 和数据缓存不一致")
            resp = requests.get(ENDPOINT_URL.format(user_id=self.uid))
            if not resp.ok:
                raise ProviderError(f"Meet network error. {resp.reason}")
            resp_data = resp.json()
            if resp_data["result_code"] != 200:
                raise ProviderError(f"Unexpected error. {data}")

            body = resp_data["data_body"]
            duration = body["durationList"]
            learn = body["learnList"]

            def today_transform(date):
                if date == "今日":
                    return datetime.today().strftime("%Y-%m-%d")
                return f"{datetime.today().year}-{date}"

            for i in duration:
                full_date = today_transform(i["date"])
                if full_date not in data["utils"]:
                    data["utils"][full_date] = {}
                dur = i["duration"]
                data["utils"][full_date]["time"] = dur

            for i in learn:
                full_date = today_transform(i["date"])
                if full_date not in data["utils"]:
                    data["utils"][full_date] = {}
                learn = i["learnNum"]
                review = i["reviewNum"]
                data["utils"][full_date]["learn"] = learn
                data["utils"][full_date]["review"] = review

        return data


class BBDCTimeProvider(BBDCProvider):
    unit = "分钟"

    def process(self, raw: Any) -> YearData:
        result = {}
        for date, day_data in raw['utils'].items():
            date = datetime.strptime(date, "%Y-%m-%d").date()
            if self.is_date_valid(date):
                time = day_data.get('time', 0)
                result[date] = time
        return result


class BBDCWordProvider(BBDCProvider):
    unit = "词"

    def process(self, raw: Any) -> YearData:
        result = {}
        for date, day_data in raw['utils'].items():
            date = datetime.strptime(date, "%Y-%m-%d").date()
            if self.is_date_valid(date):
                learn: int = day_data.get('learn', 0)
                review: int = day_data.get('review', 0)
                result[date] = learn + review
        return result



import hashlib
from dataclasses import dataclass
from datetime import timedelta
from copy import deepcopy


@dataclass
class Configs:
    data_dir: str
    output: str
    mode: str
    file_type: str
    year: int = None
    color: str = None
    server: bool = None

    def hash(self):
        a = deepcopy(self)
        a.output = None
        return hashlib.md5(str(a).encode('UTF-8')).hexdigest()


ONE_DAY = timedelta(days=1)



import base64
import json
from abc import ABC
from datetime import datetime, timedelta
from typing import Any
from urllib.parse import urlparse, parse_qs

import click
import numpy as np
import requests
import requests.utils

from yearmaps.impl.provider import Provider
from yearmaps.utils import YearData
from yearmaps.utils.colors import indigo
from yearmaps.utils.error import ProviderError


class MiFitProvider(Provider, ABC):
    id = 'mifit'
    name = '小米运动'
    color = indigo

    def __init__(self, phone: str, password: str):
        self.phone = phone
        self.password = password
        self.user_id = None
        self.app_token = None
        self.up_token = None

    def init(self):
        with self.data_file() as data:
            if "user_id" in data and "up_token" in data:
                self.user_id = data["user_id"]
                self.up_token = data["up_token"]
                try:
                    self.login_with_token()  # check token
                    return
                except ProviderError as e:
                    if e.status_code != 400:
                        raise e
            self.login_with_password(self.phone, self.password)
            self.login_with_token()
            data["user_id"] = self.user_id
            data["up_token"] = self.up_token

    @staticmethod
    @click.command('mifit', help='小米运动')
    @click.option('--phone', '-u', type=str, required=True, help='手机号（华米健康）')
    @click.option('--password', '-p', type=str, required=True, help='密码')
    @click.option('--type', '-t', 'gtype', type=click.Choice(['sleep']), default='sleep', help='图数据类型')
    @click.pass_context
    def command(ctx: click.Context, phone: str, password: str, gtype: str):
        if gtype == 'sleep':
            provider = MiFitSleepProvider(phone, password)
        else:
            raise ProviderError(f"Unsupported type {gtype}")
        provider.render(ctx.obj)

    def login_with_password(self, phone: str, password: str):
        req_data = (
            ("phone_number", phone),
            ("password", password),
            ("state", "REDIRECTION"),
            ("client_id", "HuaMi"),
            ("country_code", "CN"),
            ("token", "access"),
            ("token", "refresh"),
            ("redirect_uri", "https://s3-us-west-2.amazonaws.com/hm-registration/successsignin.html"),
        )
        resp = requests.post(f"https://api-user.huami.com/registrations/%2B86{phone}/tokens", data=req_data,
                             allow_redirects=False)
        if not resp.ok:
            raise ProviderError(f"Login failed {resp.status_code} {resp.reason}")
        location = resp.headers["Location"]
        query = urlparse(location).query
        self.up_token = parse_qs(query)["access"][0]  # should be cached

    def login_with_token(self):
        req_data = {
            "app_name": "com.xiaomi.hm.health",
            "country_code": "CN",
            "code": self.up_token,
            "device_id": "02:00:00:00:00:00",
            "device_model": "android_phone",
            "app_version": "4.0.17",
            "grant_type": "access_token",
            "allow_registration": "false",
            "dn": "account.huami.com,api-user.huami.com,api-watch.huami.com,"
                  "api-analytics.huami.com,app-analytics.huami.com,api-mifit.huami.com",
            "third_name": "huami_phone",
            "source": "com.xiaomi.hm.health:4.0.17:50283",
            "lang": "zh"
        }
        resp = requests.post("https://account.huami.com/v2/client/login", data=req_data)
        if not resp.ok:
            err = ProviderError(f"Login failed {resp.status_code} {resp.reason}")
            err.status_code = resp.status_code
            raise err
        resp_data = resp.json()
        self.app_token = resp_data["token_info"]["app_token"]
        self.user_id = resp_data["token_info"]["user_id"]


class MiFitSleepProvider(MiFitProvider):
    name = '睡眠时间'

    @staticmethod
    def format_time(time: int) -> str:
        t = timedelta(seconds=time)
        return f"平均：{t.seconds // 3600} 小时 {t.seconds // 60 % 60} 分钟"

    unit = format_time

    @staticmethod
    def label_format(value: int) -> str:
        t = timedelta(seconds=value)
        hours = str(t.seconds // 3600).zfill(2)
        minutes = str(t.seconds // 60 % 60).zfill(2)
        return f"{hours}:{minutes}"

    @staticmethod
    def analysis(data: np.ndarray):
        return np.nanmean(data)

    def access(self) -> Any:
        headers = {
            "timezone": "Asia/Shanghai",
            "apptoken": self.app_token,
            "country": "CN",
            "appplatform": "android_phone",
            "appname": "com.xiaomi.hm.health",
        }
        params = {
            "userid": self.user_id,
            "country": "CN",
            "device": "android_30",
            "device_type": "android_phone",
            "lang": "zh_CN",
            "query_type": "summary",
            "timezone": "Asia/Shanghai",
            "from_date": self.start_date().strftime("%Y-%m-%d"),
            "to_date": self.end_date().strftime("%Y-%m-%d"),
        }
        resp = requests.get("https://api-mifit.huami.com/v1/data/band_data.json", headers=headers, params=params)
        if not resp.ok:
            raise ProviderError(f"Access failed {resp.status_code} {resp.reason}")
        return resp.json()

    def process(self, raw: Any) -> YearData:
        ret = {}
        data = raw["data"]
        for piece in data:
            date = datetime.strptime(piece["date_time"], "%Y-%m-%d").date()
            summary = json.loads(base64.b64decode(piece["summary"]).decode("utf-8"))
            sleep = summary['slp']
            value = sleep['ed'] - sleep['st']
            if value > 3600 * 3:  # 3 hour
                ret[date] = value
        return ret



import os

import click

from yearmaps.constant import Configs
from yearmaps.provider import providers
from yearmaps.utils.colors import color_list
from yearmaps.utils.file import default_data_dir


@click.group()
@click.option('--data-dir', '-d', default=str(default_data_dir()), type=str, show_default=True,
              help='Directory to store datas')
@click.option('--output-dir', '-o', default=os.getcwd(), type=str, show_default=True,
              help='Directory to store output')
@click.option('--file-type', '-f', default='svg', type=click.Choice(['svg', 'png']), show_default=True,
              help='File type to export')
@click.option('--mode', '-m', default='till_now', type=click.Choice(['till_now', 'year']), show_default=True,
              help='Generate mode of the program')
@click.option('--year', '-y', type=int,
              help='Year to generate, this option will override mode to "year"')
@click.option('--color', '-c', type=click.Choice(color_list.keys()),
              help='Color to override provider default color')
@click.pass_context
def cli(ctx: click.Context, data_dir: str, output_dir: str, file_type: str, mode: str, year: int, color: str):
    ctx.obj = Configs(data_dir=data_dir, output=output_dir, mode=mode, file_type=file_type, color=color)
    obj = ctx.obj
    if mode == 'year':
        if year is None:
            raise click.BadParameter('Year is required when mode is "year"')
        obj.year = year
    elif mode == 'till_now':
        if year is not None:
            click.echo('Year is not ignored when mode is "till_now"', err=True)
    click.echo("Selecting provider.")


def main():
    for provider in providers:
        cli.add_command(provider.command)
    cli()


if __name__ == '__main__':
    main()



import os
import threading
import time
from datetime import datetime
from pathlib import Path
from typing import Dict, List

import click
import indexpy
import schedule
import uvicorn
import yaml
from indexpy import request

from yearmaps.constant import Configs
from yearmaps.impl.task import Task
from yearmaps.provider import providers, provider_map
from yearmaps.utils import file
from yearmaps.utils.colors import color_list
from yearmaps.utils.file import ensure_dir
from yearmaps.utils.util import option_name


@click.command()
@click.option('--host', '-l', default='0.0.0.0', show_default=True, type=str, help='Host to listen on.')
@click.option('--port', '-p', default=5000, show_default=True, type=int, help='Port to listen on.')
@click.option('--config', '-f', default=str(Path(os.getcwd()) / 'yearmaps.yml'), type=str, show_default=True,
              help='Path to config file.')
@click.pass_context
def cli(ctx: click.Context, host: str, port: int, config: str):
    # Load config
    if not Path(config).resolve().exists():
        raise FileNotFoundError(f'Config file not found: {config}')
    with open(config, 'r', encoding='UTF-8') as f:
        config_dict: Dict = yaml.load(f, Loader=yaml.FullLoader)

    # Initialize server args
    # has default value from click
    if host is not None:
        config_dict['host'] = host
    if port is not None:
        config_dict['port'] = port

    # Initialize config from yaml
    if 'data-dir' not in config_dict.keys():
        config_dict['data-dir'] = file.default_data_dir()
    if 'cache-dir' not in config_dict.keys():
        config_dict['cache-dir'] = file.default_cache_dir()
    if 'mode' not in config_dict.keys():
        config_dict['mode'] = 'till_now'
    if 'year' not in config_dict.keys() and config_dict['mode'] == 'year':
        config_dict['year'] = datetime.now().year
    if 'file_type' not in config_dict.keys():
        config_dict['file_type'] = 'png'

    # Check dirs
    ensure_dir(config_dict['data-dir'])
    ensure_dir(config_dict['cache-dir'])

    # Copy yaml config to context object
    ctx.obj = Configs(
        data_dir=config_dict['data-dir'],
        output=config_dict['cache-dir'],  # should append filename later
        mode=config_dict['mode'],
        year=config_dict.get('year', None),
        file_type=config_dict['file_type'],
        color=config_dict.get('color', None)
    )
    obj = ctx.obj
    obj.server = True

    # Check provider config
    if 'providers' not in config_dict.keys():
        raise KeyError('Providers not found in config file.')

    if not isinstance(config_dict['providers'], Dict):
        raise TypeError('Providers must have arguments.')

    # Build required params for each provider
    provider_required_params = {}
    for provider in providers:
        command: click.Command = provider.command
        params = command.params
        name = command.name
        provider_required_params[name] = []
        for param in params:
            if param.required:
                provider_required_params[name].append(param.name)

    # Validate provider config
    for key in config_dict['providers'].keys():
        if key not in provider_required_params:
            raise KeyError(f'Provider not found: {key}')

        provider_config = config_dict['providers'][key]
        if not isinstance(provider_config, Dict):
            raise TypeError(f'Provider {key} must be a dict.')

    def command_option_map(com: click.Command, option: str) -> str:
        for par in com.params:
            if option == par.name:
                return option
            for opt in par.opts:
                if option_name(opt) == option:
                    return par.name
        raise ValueError(f'Option not found: {option}')

    # Build tasks
    task_list: List[Task] = []
    for provider_key, provider_config in config_dict['providers'].items():
        global_config = {}

        provider = provider_map[provider_key]
        command = provider.command
        name = provider.name

        if 'global' in provider_config.keys():
            if not isinstance(provider_config['global'], Dict):
                raise TypeError('Global config must be a dict.')
            for key, value in provider_config['global'].items():
                # Additional check for global config
                if not hasattr(obj, key):
                    raise KeyError(f'{key} is not a valid global option.')

                if key == 'mode':
                    if value not in ['till_now', 'year']:
                        raise ValueError(f'{value} is not a valid mode.')
                elif key == 'year':
                    if not isinstance(value, int):
                        raise TypeError('Year must be an integer.')
                    if value < 2000 or value > datetime.now().year:
                        raise ValueError(f'{value} is not a valid year.')
                elif key == 'file_type':
                    if value not in ['png', 'svg']:
                        raise ValueError(f'{value} is not a supported file type.')
                elif key == 'color':
                    if not isinstance(value, str):
                        raise TypeError('Color must be a string.')
                    if value not in color_list:
                        raise ValueError(f'{value} is not a valid color.')

                global_config[key] = value
                provider_config.pop('global')

        replace_keys = []
        for key, value in provider_config.items():
            mapped_option = command_option_map(command, key)
            if key != mapped_option:
                replace_keys.append((key, mapped_option))
        for old, new in replace_keys:
            provider_config[new] = provider_config.pop(old)

        if not set(provider_config.keys()).issuperset(set(provider_required_params[provider_key])):
            raise KeyError(
                'Provider {key} is missing required params: '
                f'{set(provider_required_params[provider_key]) - set(provider_config.keys())}')

        task_list.append(Task(name, ctx, command, global_config, provider_config))

    def ensure_cache():
        for queue_task in task_list:
            queue_task.ensure_cache(quiet=True)

    threading.Thread(target=ensure_cache).start()

    def update_cache():
        for queue_task in task_list:
            queue_task.update_cache()

    schedule.every().day.at('00:00').do(update_cache)

    def loop():
        while True:
            wait_time = schedule.idle_seconds()
            time.sleep(wait_time)
            schedule.run_pending()

    regen = threading.Thread(target=loop)
    regen.daemon = True
    regen.start()

    # Build tasks hash table
    tasks_hash_table = {}
    for task in task_list:
        click.echo(f'Valid task: {task.task_name()}')
        tasks_hash_table[task.task_hash()] = task

    @app.router.http('/', name='index')
    @app.router.http('/{path:any}', name='route')
    async def static():
        if request.path_params:
            path = request.path_params['path']
        else:
            path = 'index.html'

        if path == 'api':
            ret = []
            for task_hash, task_item in tasks_hash_table.items():
                ret.append((task_item.command.name, task_item.name, task_hash))
            return indexpy.JSONResponse(ret)

        if path in tasks_hash_table:
            return indexpy.FileResponse(filepath=str(tasks_hash_table[path].cache_path()))

        static_file = Path(__file__).parent / "static" / path
        if static_file.is_file():
            return indexpy.FileResponse(filepath=str(static_file))
        return indexpy.HttpResponse(status_code=404)

    # noinspection PyTypeChecker
    uvicorn.run(app, host=config_dict['host'], port=config_dict['port'])


app = indexpy.Index()

if __name__ == '__main__':
    cli()



from yearmaps.provider.BBDCProvider import BBDCProvider
from yearmaps.provider.BilibiliProvider import BilibiliProvider
from yearmaps.provider.CodeforcesProvider import CodeforcesProvider
from yearmaps.provider.GitHubProvider import GitHubProvider
from yearmaps.provider.MiFitProvider import MiFitProvider

providers = [BBDCProvider, BilibiliProvider, CodeforcesProvider, GitHubProvider, MiFitProvider]

provider_map = {x.command.name: x for x in providers}



from abc import ABC
from collections import defaultdict
from datetime import date
from typing import Any

import click
import requests

from yearmaps.impl.provider import Provider
from yearmaps.utils import YearData
from yearmaps.utils.colors import pink
from yearmaps.utils.error import ProviderError

ENDPOINT_URL = "https://api.bilibili.com/x/space/arc/search?mid={uid}&ps=50&pn={pn}"


class BilibiliProvider(Provider, ABC):
    name = "bilibili"
    id = "bili"
    color = pink

    def __init__(self, uid: str):
        self.uid = uid

    def access(self) -> Any:
        data_ret = []
        pn = 1
        data_tmp = requests.get(ENDPOINT_URL.format(uid=self.uid, pn=pn)).json()
        ps = data_tmp["data"]["page"]["ps"]
        cnt = data_tmp["data"]["page"]["count"]
        while (pn - 1) * ps <= cnt:
            data_ret.extend(data_tmp["data"]["list"]["vlist"])
            pn += 1
            data_tmp = requests.get(ENDPOINT_URL.format(uid=self.uid, pn=pn)).json()
        return data_ret

    @staticmethod
    @click.command("bili", help="Bilibili")
    @click.option("--id", "-i", "uid", type=str, required=True, help="bilibili uid")
    @click.option("--type", "-t", "gtype", type=click.Choice(("video",)), default="video", help="图数据类型")
    @click.pass_context
    def command(ctx: click.Context, uid: str, gtype: str):
        if gtype == "video":
            provider = BilibiliVideoProvider(uid)
        else:
            raise ProviderError(f"Unknown type {gtype}")
        provider.render(ctx.obj)


class BilibiliVideoProvider(BilibiliProvider):
    unit = "Video"

    def process(self, raw: Any) -> YearData:
        d = defaultdict(int)
        for res in raw:
            d[date.fromtimestamp(res["created"])] += 1
        return d



import hashlib
import json


def dict_hash(d: dict) -> str:
    return hashlib.md5(json.dumps(d, sort_keys=True).encode('UTF-8')).hexdigest()


def str_hash(*s: str) -> str:
    return hashlib.md5(("".join(s)).encode('UTF-8')).hexdigest()


def option_name(name: str) -> str:
    if name.startswith('--'):
        name = name[2:]
    elif name.startswith('-'):
        name = name[1:]
    name = name.replace('-', '_')
    name = name.lower()
    return name



import tempfile
from pathlib import Path
from typing import Union

from yearmaps.utils.error import ProviderError


def ensure_dir(dir_: Union[Path, str]) -> Path:
    if isinstance(dir_, str):
        dir_ = Path(dir_)
    if not dir_.exists():
        if not dir_.parent.exists():
            raise FileExistsError(f"Parent directory of {dir_.absolute().__str__()} does not exist")
        dir_.mkdir()
        return dir_
    if not dir_.is_dir():
        raise ProviderError(f"{dir_.absolute().__str__()} is not a directory")
    return dir_


def default_data_dir() -> Path:
    home = Path.home()
    if not home.exists():
        raise ProviderError("No home directory found")
    data_dir = home / ".yearmaps"
    return ensure_dir(data_dir)


def default_cache_dir() -> Path:
    return Path(tempfile.gettempdir()) / "yearmaps-cache"



from typing import List

import click
import numpy as np
from matplotlib.colors import ListedColormap, to_rgba_array

no_value_color = to_rgba_array("#f6f6f6")


class PolarisationColorMap(ListedColormap):
    def __init__(self, colors: List[str], color_need: int, no_grey_index: List[int]):
        colors = colors.copy()
        self.zero_color = to_rgba_array(colors[0])
        self.no_grey_index = no_grey_index
        colors.pop(0)
        if color_need != -1:
            if color_need == 0:
                colors = colors[:1]
            else:
                colors = colors[:color_need]
        super().__init__(colors)

    # noinspection PyShadowingBuiltins
    def __call__(self, X, alpha=None, bytes=False):
        if isinstance(X, np.ma.core.MaskedArray):
            index = np.where(X == 0)
            mask_index = np.where(X.mask)
            color = super(ListedColormap, self).__call__(X, alpha=alpha, bytes=bytes)
            for i in index:
                color[i] = self.zero_color
            for i in mask_index[0]:
                if i in self.no_grey_index:
                    continue
                color[i] = no_value_color
            return color
        click.echo(f"Unhandled situation {type(X)}", err=True)
        return super(ListedColormap, self).__call__(X, alpha=alpha, bytes=bytes)


class WithBlankListedColorMap(ListedColormap):  # TODO: support color need
    def __init__(self, colors: List[str], no_grey_index: List[int]):
        self.no_grey_index = no_grey_index
        super().__init__(colors)

    # noinspection PyShadowingBuiltins
    def __call__(self, X, alpha=None, bytes=False):
        if isinstance(X, np.ma.core.MaskedArray):
            mask_index = np.where(X.mask)
            color = super(ListedColormap, self).__call__(X, alpha=alpha, bytes=bytes)
            for i in mask_index[0]:
                if i in self.no_grey_index:
                    continue
                color[i] = no_value_color
            return color
        click.echo(f"Unhandled situation {type(X)}", err=True)
        return super(ListedColormap, self).__call__(X, alpha=alpha, bytes=bytes)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yearmaps',
 'yearmaps.impl',
 'yearmaps.interface',
 'yearmaps.provider',
 'yearmaps.utils']

package_data = \
{'': ['*'],
 'yearmaps': ['static/*', 'static/static/css/*', 'static/static/js/*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'click>=8.0.3,<9.0.0',
 'index.py>=0.21.11,<0.22.0',
 'matplotlib>=3.5.1,<4.0.0',
 'numpy>=1.22.1,<2.0.0',
 'requests>=2.27.1,<3.0.0',
 'schedule>=1.1.0,<2.0.0',
 'uvicorn>=0.17.0,<0.18.0']

entry_points = \
{'console_scripts': ['yearmaps = yearmaps.script:main',
                     'yearmaps-server = yearmaps.server:cli']}

setup_kwargs = {
    'name': 'yearmaps',
    'version': '0.1.0',
    'description': 'Generate heat map of a year.',
    'long_description': '# YearMaps\n\n![PyPI](https://img.shields.io/pypi/v/yearmaps) ![Docker Image Version (latest by date)](https://img.shields.io/docker/v/zxilly/yearmaps)\n\n生成一年的热力图。\n\n![image](https://user-images.githubusercontent.com/31370133/150357084-f0ddb8f5-26c0-4526-9f3e-bc1e3aa1784a.png)\n\n# 安装\n\n```bash\npip[3] install --user yearmaps\n```\n\n# 使用\n\n```bash\nUsage: yearmaps [OPTIONS] COMMAND [ARGS]...\n\nOptions:\n  -d, --data-dir TEXT             Directory to store datas  [default: ~/.yearmaps]\n  -o, --output-dir TEXT           Directory to store output  [default: $(pwd)]\n  -f, --file-type [svg|png]       File type to export  [default: svg]\n  -m, --mode [till_now|year]      Generate mode of the program  [default: till_now]\n  -y, --year INTEGER              Year to generate, this option will override mode to "year"\n  -c, --color [red|pink|purple|deeppurple|indigo|blue|lightblue|cyan|teal|green|lightgreen|lime|yellow|amber|orange|deeporange|brown|grey|bluegrey]\n                                  Color to override provider default color\n  --help                          Show this message and exit.\n\nCommands:\n  bbdc    不背单词\n  bili    Bilibili\n  cf      Codeforces\n  github  GitHub\n  \n# 服务器模式\n\nUsage: yearmaps-server [OPTIONS]\n\nOptions:\n  -l, --host TEXT     Host to listen on.  [default: 0.0.0.0]\n  -p, --port INTEGER  Port to listen on.  [default: 5000]\n  -f, --config TEXT   Path to config file.  [default: $(pwd)/yearmaps.yml]\n  --help              Show this message and exit.\n```\n\n## 子模块\n\n### 不背单词\n\n<details>\n\n```bash\nUsage: yearmaps bbdc [OPTIONS]\n\n  不背单词\n\nOptions:\n  -i, --id  TEXT          不背单词用户 ID  [required]\n  -t, --type [time|word]  图数据类型\n  --help                  Show this message and exit.\n```\n\n![bbdc](https://user-images.githubusercontent.com/31370133/150357416-36b3bd83-aa8c-4065-aabb-f130f0392476.png)\n\n</details>\n\n### bilibili\n\n<details>\n  \n```bash\nUsage: yearmaps bili [OPTIONS]\n\n  bilibili\n\nOptions:\n  -i, --id TEXT       bilibili uid  [required]\n  -t, --type [video]  图数据类型\n  --help              Show this message and exit.\n```\n  \n![image](https://user-images.githubusercontent.com/50107074/150572220-781dd51f-fd9c-47cf-b78a-cac1def2fd91.png)\n  \n</details>\n\n### Codeforces\n\n<details>\n\n```bash\nUsage: yearmaps cf [OPTIONS]\n\n  Codeforces\n\nOptions:\n  -u, --user TEXT      Codeforces user name  [required]\n  -t, --type [all|ac]  图数据类型\n  --help               Show this message and exit.\n```\n\n\n![image](https://user-images.githubusercontent.com/31370133/150477193-6740583e-f3b8-48a3-b92c-f40b4af010b8.png)\n\n</details>\n\n### GitHub\n\n<details>\n\n```bash\nUsage: yearmaps github [OPTIONS]\n\n  GitHub\n\nOptions:\n  -u, --user TEXT       GitHub user name  [required]\n  -k, --token TEXT      GitHub access token  [required]\n  -t, --type [contrib]  图数据类型\n  --help                Show this message and exit.\n```\n\n![image](https://user-images.githubusercontent.com/31370133/150357084-f0ddb8f5-26c0-4526-9f3e-bc1e3aa1784a.png)\n\n</details>\n',
    'author': 'Zxilly',
    'author_email': 'zxilly@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/zxilly/yearmaps',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import calendar
import datetime
from abc import ABC
from pathlib import Path

import matplotlib as mpl
import matplotlib.axes
import matplotlib.figure
import numpy as np
from matplotlib import pyplot as plt
from matplotlib.axes import Axes
from matplotlib.ticker import ScalarFormatter
from matplotlib.transforms import Bbox

from yearmaps.constant import Configs, ONE_DAY
from yearmaps.interface.provider import ProviderInterface, date_range, ProviderUtils
from yearmaps.utils.colors import color_list
from yearmaps.utils.colormap import PolarisationColorMap, WithBlankListedColorMap


class Provider(ProviderInterface, ProviderUtils, ABC):

    # Render utils to output
    def render(self, options: Configs):
        self.options = options
        self.echo(f"Initializing {self.name} provider...")
        self.init()
        self.echo(f"Initialized {self.name} provider.")
        self.echo(f"Start access {self.name} data...")
        raw = self.access()
        self.echo("End access data.")
        self.echo(f"Start process {self.name} data...")
        data = self.process(raw)
        self.echo("End process data.")

        def fulfill_data() -> np:
            start = self.start_date()
            end = self.end_date()

            graph_start = start
            graph_end = end

            while graph_start.weekday() != 0:
                graph_start -= ONE_DAY
            while graph_end.weekday() != 6:
                graph_end += ONE_DAY

            days = (graph_end - graph_start).days
            weeks = days // 7
            if weeks * 7 < days:
                weeks += 1

            empty_grid = np.empty((7, weeks), dtype="float64")
            empty_grid = np.nan * empty_grid

            def get_day(i_: datetime.date) -> int:
                return i_.weekday()

            def get_week(i_: datetime.date) -> int:
                return (i_ - graph_start).days // 7

            _no_grey_index = []  # FIXME: too ugly implementation, remove it later

            def fill_no_grey(d: datetime.date):
                day = get_day(d)
                week = get_week(d)
                index = day * weeks + week
                _no_grey_index.append(index)

            for date in date_range(graph_start, start - ONE_DAY):
                fill_no_grey(date)
            for date in date_range(end + ONE_DAY, graph_end):
                fill_no_grey(date)

            for date, value in data.items():
                if not self.is_date_valid(date):
                    continue

                empty_grid[get_day(date)][get_week(date)] = value

            month_list = np.linspace(-1, -1, weeks, dtype=int)
            year_tuple = (start.year, get_week(start))
            current_year = None
            for date in date_range(start, end):
                date: datetime.date
                if date.weekday() == 0:
                    month_list[get_week(date)] = date.month
                if date.weekday() == 6:
                    if current_year is None:
                        current_year = date.year
                    elif current_year != date.year:
                        if year_tuple[0] != date.year:
                            year_tuple = (date.year, get_week(date))
            return empty_grid, month_list, year_tuple, _no_grey_index

        grid, months, (year, year_loc), no_grey_index = fulfill_data()
        grid: np.ndarray

        mpl.rcParams['font.family'] = 'monospace'
        mpl.rcParams['svg.fonttype'] = 'none'
        mpl.rcParams['font.sans-serif'] = ['Microsoft YaHei', 'Noto Sans CJK SC', 'Noto Sans CJK JP', 'Noto Sans CJK']

        grid_max = self.value_type(np.nanmax(grid))
        grid_min = self.value_type(np.nanmin(grid))

        if grid_max == grid_min:
            raise ValueError("No data to collected.")

        color = self.options.color
        if color is None:
            color = self.color
        else:
            color = color_list[color]

        if self.value_type == int:
            color_need = int(grid_max)
        else:
            color_need = -1

        if grid_min == 0:
            c_map = PolarisationColorMap(color, color_need, no_grey_index)
        else:
            c_map = WithBlankListedColorMap(color, no_grey_index)

        fig_size = (10, 3)
        fig, ax = plt.subplots(figsize=fig_size, dpi=400)
        fig: matplotlib.figure.Figure
        ax: matplotlib.axes.Axes

        pc = ax.pcolormesh(grid, edgecolors=ax.get_facecolor(), linewidth=1, cmap=c_map)
        pc.set_clim(grid_min, grid_max)
        ax.invert_yaxis()
        ax.set_aspect("equal")

        # add weekdays label
        ax.tick_params(axis="y", which="major", pad=1, width=0, colors='#24292f')

        ax.set_yticks([x + 0.5 for x in range(1, 6, 2)])
        ax.set_yticklabels(
            ['Tue', 'Thu', 'Sat'],
        )

        # add months label
        current_month = -1
        month_locs = []
        month_labels = []
        for i, month in enumerate(months):
            if month != current_month:
                month_locs.append(i + 0.5)
                month_labels.append(calendar.month_abbr[int(month)])
                current_month = month

        # trick to remove too close labels
        if month_locs[1] - month_locs[0] < 4:
            month_locs.pop(0)
            month_labels.pop(0)
        if month_locs[-1] - month_locs[-2] < 4:
            month_locs.pop()
            month_labels.pop()

        ax.tick_params(axis="x", which="major", pad=1, width=0, color='#24292f')
        ax.set_xticks(month_locs)
        ax.set_xticklabels(month_labels, ha="center")
        ax.xaxis.tick_top()

        if self.options.mode == 'till_now':
            se_cax = ax.secondary_xaxis('bottom')
            se_cax.set_xticks([year_loc])
            se_cax.set_xticklabels([year], ha="left")
            se_cax.tick_params(axis="x", pad=0, width=0, color='#24292f')
            se_cax.set_frame_on(False)

        # Remove the axis spines
        ax.set_frame_on(False)

        bbox: Bbox = ax.get_position()
        cax: Axes = fig.add_axes(
            [
                bbox.x1 + 0.015,
                bbox.y0,
                0.015,
                bbox.height
            ]
        )
        cax.set_frame_on(False)
        plt.colorbar(pc, cax=cax, format=ScalarFormatter())

        font_family = 'sans-serif'
        hint_font_dict = {
            'fontfamily': font_family,
        }
        label_font_dict = {
            'fontfamily': 'monospace',
        }

        # Color bar label
        offset = grid_max - grid_min
        cax.set_yticks([grid_min + offset / 9, grid_max - offset / 9])
        cax.set_yticklabels(labels=[self.label_format(grid_min), self.label_format(grid_max)], fontdict=label_font_dict)
        cax.tick_params(axis="y", which="major", pad=0, width=0)

        title_font_dict = {'fontsize': 30,
                           'fontfamily': font_family,
                           'fontweight': 'bold'}
        ax.set_title(self.name, fontdict=title_font_dict, pad=15, loc='left')

        year_font_dict = {
            **hint_font_dict,
            'fontsize': 28,
            'color': '#A9A9A9',
            'fontweight': 'bold'
        }

        if isinstance(self.unit, str):
            analysis = f"{self.value_type(self.analysis(grid))} {self.unit}"
        else:
            analysis = self.unit(self.value_type(self.analysis(grid)))

        # overall analysis
        ax.text(1, 1.25, analysis,
                horizontalalignment='right',
                verticalalignment='bottom',
                fontdict=hint_font_dict,
                transform=ax.transAxes)

        # year analysis on the left
        if self.options.mode == 'year':
            ax.text(-0.075, 0.6, f"{self.options.year}",
                    horizontalalignment='center',
                    verticalalignment='center',
                    fontdict=year_font_dict,
                    rotation=90,
                    transform=ax.transAxes)

        # save the figure
        file_type = self.options.file_type

        if self.options.server:
            path = Path(self.options.output)
        else:
            path = Path(self.options.output) / f"{self.id}.{file_type}"
        plt.savefig(str(path), bbox_inches='tight', pad_inches=0.1, format=file_type)



from abc import ABC
from collections import defaultdict
from datetime import date
from typing import Any

import click
import requests

from yearmaps.impl.provider import Provider
from yearmaps.utils import YearData
from yearmaps.utils.colors import purple
from yearmaps.utils.error import ProviderError

ENDPOINT_URL = "https://codeforces.com/api/user.status?handle={user}"


class CodeforcesProvider(Provider, ABC):
    name = "Codeforces"
    id = "cf"
    color = purple

    def __init__(self, user: str):
        self.user = user

    def access(self) -> Any:
        return requests.get(ENDPOINT_URL.format(user=self.user)).json()

    @staticmethod
    @click.command("cf", help="Codeforces")
    @click.option("--user", "-u", type=str, required=True, help="Codeforces user name")
    @click.option("--type", "-t", "gtype", type=click.Choice(("all", "ac")), default="all", help="图数据类型")
    @click.pass_context
    def command(ctx: click.Context, user: str, gtype: str):
        if gtype == "all":
            provider = CodeforcesAllProvider(user)
        elif gtype == "ac":
            provider = CodeforcesACProvider(user)
        else:
            raise ProviderError(f"Unknown type {gtype}")
        provider.render(ctx.obj)


class CodeforcesACProvider(CodeforcesProvider):
    unit = "Accepted Problems"

    def process(self, raw: Any) -> YearData:
        d = defaultdict(int)
        for res in raw["result"]:
            if res["verdict"] == "OK":
                d[date.fromtimestamp(res["creationTimeSeconds"])] += 1
        return d


class CodeforcesAllProvider(CodeforcesProvider):
    unit = "Problems"

    def process(self, raw: Any) -> YearData:
        d = defaultdict(int)
        for res in raw["result"]:
            d[date.fromtimestamp(res["creationTimeSeconds"])] += 1
        return d



from abc import ABC
from datetime import datetime
from typing import Any

import click
import requests

from yearmaps.impl.provider import Provider
from yearmaps.utils import YearData
from yearmaps.utils.colors import blue
from yearmaps.utils.error import ProviderError

ENDPOINT_URL = "https://api.github.com/graphql"

contribution_query = """
query ($user: String!,$start: DateTime) {
  user(login: $user) {
    contributionsCollection(from:$start) {
      contributionCalendar {
        weeks {
          contributionDays {
            contributionCount
            date
          }
        }
      }
    }
  }
}
"""


class GitHubProvider(Provider, ABC):
    id = "github"
    name = "GitHub"
    color = blue

    def __init__(self, user: str, token: str):
        self.user = user
        self.token = token

    @staticmethod
    @click.command('github', help="GitHub")
    @click.option('--user', '-u', type=str, required=True, help="GitHub user name")
    @click.option("--token", '-k', type=str, required=True, help="GitHub access token")
    @click.option('--type', '-t', 'gtype', type=click.Choice(['contrib']), default='contrib', help="图数据类型")
    @click.pass_context
    def command(ctx: click.Context, user: str, gtype: str, token: str):
        if gtype == 'contrib':
            provider = GitHubContribProvider(user, token)
        else:
            raise ProviderError(f"Unknown type {gtype}")
        provider.render(ctx.obj)


class GitHubContribProvider(GitHubProvider):
    unit = "Contribution"

    def access(self) -> Any:
        resp = requests.post(ENDPOINT_URL, json={
            'query': contribution_query,
            'variables': {
                'user': self.user,
                'start': datetime.combine(self.start_date(), datetime.min.time()).isoformat()
            }
        }, headers={
            'Authorization': f"bearer {self.token}"
        })
        if not resp.ok:
            raise ProviderError(f"{resp.status_code} - {resp.reason}")
        return resp.json()

    def process(self, raw: Any) -> YearData:
        result = {}
        data = raw['data']['user']['contributionsCollection']['contributionCalendar']['weeks']
        for week in data:
            for day in week['contributionDays']:
                date = datetime.fromisoformat(day['date']).date()
                value = day['contributionCount']
                result[date] = value
        return result
